#!/usr/bin/python
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#
#

#
# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
#

class ApiException(Exception):
        """Base exception class for all server.api exceptions."""
        def __init__(self, *args):
                Exception.__init__(self, *args)
                if args:
                        self.data = args[0]

        def __unicode__(self):
                # To workaround python issues 6108 and 2517, this provides a
                # a standard wrapper for this class' exceptions so that they
                # have a chance of being stringified correctly.
                return str(self)

        def __str__(self):
                return str(self.data)


class VersionException(ApiException):
        """Exception used to indicate that the client's requested api version
        is not supported.
        """
        def __init__(self, expected_version, received_version):
                ApiException.__init__(self)
                self.expected_version = expected_version
                self.received_version = received_version

        def __str__(self):
                return "Incompatible API version '{0}' specified; " \
                    "expected: '{1}'.".format(self.received_version,
                    self.expected_version)


class RedirectException(ApiException):
        """Used to indicate that the client should be redirected to a new
        URI.
        """
        pass


class UnrecognizedOptionsToInfo(ApiException):
        def __init__(self, opts):
                ApiException.__init__(self)
                self._opts = opts

        def __str__(self):
                s = _("Info does not recognize the following options: {0}").format(
                    ", ".join(str(o) for o in self._opts))
                return s
